@@ -13,9 +13,9 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-""" 
+"""
 Acknowledgements: Many of the modeling parts here come from the great transformers repository: https://github.com/huggingface/transformers.
-Thanks for the great work! 
+Thanks for the great work!
 """
 
 from typing import Type, Optional, Dict, Any, Union, List
@@ -26,7 +26,6 @@ import logging
 import os
 from abc import ABC, abstractmethod
 from pathlib import Path
-from functools import wraps
 import numpy as np
 import torch
 from torch import nn
@@ -36,6 +35,7 @@ from transformers import AutoModel, AutoConfig
 from transformers.modeling_utils import SequenceSummary
 
 from haystack.errors import ModelingError
+from haystack.modeling.utils import silence_transformers_logs
 
 
 logger = logging.getLogger(__name__)
@@ -58,30 +58,6 @@ LANGUAGE_HINTS = (
 OUTPUT_DIM_NAMES = ["dim", "hidden_size", "d_model"]
 
 
-def silence_transformers_logs(from_pretrained_func):
-    """
-    A wrapper that raises the log level of Transformers to
-    ERROR to hide some unnecessary warnings.
-    """
-
-    @wraps(from_pretrained_func)
-    def quiet_from_pretrained_func(cls, *args, **kwargs):
-
-        # Raise the log level of Transformers
-        t_logger = logging.getLogger("transformers")
-        original_log_level = t_logger.level
-        t_logger.setLevel(logging.ERROR)
-
-        result = from_pretrained_func(cls, *args, **kwargs)
-
-        # Restore the log level
-        t_logger.setLevel(original_log_level)
-
-        return result
-
-    return quiet_from_pretrained_func
-
-
 # TODO analyse if LMs can be completely used through HF transformers
 class LanguageModel(nn.Module, ABC):
     """
@@ -155,7 +131,7 @@ class LanguageModel(nn.Module, ABC):
         with open(save_filename, "w") as file:
             file.write(string)
 
-    def save(self, save_dir: Union[str, Path], state_dict: Dict[Any, Any] = None):
+    def save(self, save_dir: Union[str, Path], state_dict: Optional[Dict[Any, Any]] = None):
         """
         Save the model `state_dict` and its configuration file so that it can be loaded again.
 
@@ -172,7 +148,7 @@ class LanguageModel(nn.Module, ABC):
         self.save_config(save_dir)
 
     def formatted_preds(
-        self, logits, samples, ignore_first_token: bool = True, padding_mask: torch.Tensor = None
+        self, logits, samples, ignore_first_token: bool = True, padding_mask: Optional[torch.Tensor] = None
     ) -> List[Dict[str, Any]]:
         """
         Extracting vectors from a language model (for example, for extracting sentence embeddings).
@@ -267,7 +243,7 @@ class HFLanguageModel(LanguageModel):
         self,
         pretrained_model_name_or_path: Union[Path, str],
         model_type: str,
-        language: str = None,
+        language: Optional[str] = None,
         n_added_tokens: int = 0,
         use_auth_token: Optional[Union[str, bool]] = None,
         model_kwargs: Optional[Dict[str, Any]] = None,
@@ -382,7 +358,7 @@ class HFLanguageModelWithPooler(HFLanguageModel):
         self,
         pretrained_model_name_or_path: Union[Path, str],
         model_type: str,
-        language: str = None,
+        language: Optional[str] = None,
         n_added_tokens: int = 0,
         use_auth_token: Optional[Union[str, bool]] = None,
         model_kwargs: Optional[Dict[str, Any]] = None,
@@ -510,7 +486,7 @@ class DPREncoder(LanguageModel):
         self,
         pretrained_model_name_or_path: Union[Path, str],
         model_type: str,
-        language: str = None,
+        language: Optional[str] = None,
         n_added_tokens: int = 0,
         use_auth_token: Optional[Union[str, bool]] = None,
         model_kwargs: Optional[Dict[str, Any]] = None,
@@ -846,7 +822,7 @@ def get_language_model_class(model_type: str) -> Optional[Type[Union[HFLanguageM
 
 def get_language_model(
     pretrained_model_name_or_path: Union[Path, str],
-    language: str = None,
+    language: Optional[str] = None,
     n_added_tokens: int = 0,
     use_auth_token: Optional[Union[str, bool]] = None,
     revision: Optional[str] = None,
