@@ -21,14 +21,16 @@ from haystack.nodes.file_converter import (
     ParsrConverter,
 )
 from haystack.nodes.label_generator import PseudoLabelGenerator
-from haystack.nodes.other import Docs2Answers, JoinDocuments, RouteDocuments, JoinAnswers
+from haystack.nodes.other import Docs2Answers, JoinDocuments, RouteDocuments, JoinAnswers, DocumentMerger
 from haystack.nodes.preprocessor import BasePreProcessor, PreProcessor
+from haystack.nodes.prompt import PromptNode, PromptTemplate, PromptModel
 from haystack.nodes.query_classifier import SklearnQueryClassifier, TransformersQueryClassifier
 from haystack.nodes.question_generator import QuestionGenerator
 from haystack.nodes.ranker import BaseRanker, SentenceTransformersRanker
 from haystack.nodes.reader import BaseReader, FARMReader, TransformersReader, TableReader, RCIReader
 from haystack.nodes.retriever import (
     BaseRetriever,
+    DenseRetriever,
     DensePassageRetriever,
     EmbeddingRetriever,
     BM25Retriever,
@@ -39,6 +41,7 @@ from haystack.nodes.retriever import (
     TfidfRetriever,
     Text2SparqlRetriever,
     TableTextRetriever,
+    MultiModalRetriever,
 )
 from haystack.nodes.summarizer import BaseSummarizer, TransformersSummarizer
 from haystack.nodes.translator import BaseTranslator, TransformersTranslator
