@@ -37,31 +37,31 @@ class RAGenerator(BaseGenerator):
 
     **Example**
 
-    ```python
-    |     query = "who got the first nobel prize in physics?"
-    |
-    |     # Retrieve related documents from retriever
-    |     retrieved_docs = retriever.retrieve(query=query)
-    |
-    |     # Now generate answer from query and retrieved documents
-    |     generator.predict(
-    |        query=query,
-    |        documents=retrieved_docs,
-    |        top_k=1
-    |     )
-    |
-    |     # Answer
-    |
-    |     {'query': 'who got the first nobel prize in physics',
-    |      'answers':
-    |          [{'query': 'who got the first nobel prize in physics',
-    |            'answer': ' albert einstein',
-    |            'meta': { 'doc_ids': [...],
-    |                      'doc_scores': [80.42758 ...],
-    |                      'doc_probabilities': [40.71379089355469, ...
-    |                      'content': ['Albert Einstein was a ...]
-    |                      'titles': ['"Albert Einstein"', ...]
-    |      }}]}
+     ```python
+     query = "who got the first nobel prize in physics?"
+
+     # Retrieve related documents from retriever
+     retrieved_docs = retriever.retrieve(query=query)
+
+     # Now generate answer from query and retrieved documents
+     generator.predict(
+        query=query,
+        documents=retrieved_docs,
+        top_k=1
+     )
+
+     # Answer
+
+     {'query': 'who got the first nobel prize in physics',
+      'answers':
+          [{'query': 'who got the first nobel prize in physics',
+            'answer': ' albert einstein',
+            'meta': { 'doc_ids': [...],
+                      'doc_scores': [80.42758 ...],
+                      'doc_probabilities': [40.71379089355469, ...
+                      'content': ['Albert Einstein was a ...]
+                      'titles': ['"Albert Einstein"', ...]
+      }}]}
     ```
     """
 
@@ -80,6 +80,7 @@ class RAGenerator(BaseGenerator):
         use_gpu: bool = True,
         progress_bar: bool = True,
         use_auth_token: Optional[Union[str, bool]] = None,
+        devices: Optional[List[Union[str, torch.device]]] = None,
     ):
         """
         Load a RAG model from Transformers along with passage_embedding_model.
@@ -104,6 +105,11 @@ class RAGenerator(BaseGenerator):
                                 `transformers-cli login` (stored in ~/.huggingface) will be used.
                                 Additional information can be found here
                                 https://huggingface.co/transformers/main_classes/model.html#transformers.PreTrainedModel.from_pretrained
+
+        :param devices: List of torch devices (e.g. cuda, cpu, mps) to limit inference to specific devices.
+                        A list containing torch device objects and/or strings is supported (For example
+                        [torch.device('cuda:0'), "mps", "cuda:1"]). When specifying `use_gpu=False` the devices
+                        parameter is not used and a single cpu device is used for inference.
         """
         super().__init__(progress_bar=progress_bar)
 
@@ -118,11 +124,16 @@ class RAGenerator(BaseGenerator):
 
         if top_k > self.num_beams:
             top_k = self.num_beams
-            logger.warning(f"top_k value should not be greater than num_beams, hence setting it to {num_beams}")
+            logger.warning("top_k value should not be greater than num_beams, hence setting it to %s", num_beams)
 
         self.top_k = top_k
 
-        self.devices, _ = initialize_device_settings(use_cuda=use_gpu, multi_gpu=False)
+        self.devices, _ = initialize_device_settings(devices=devices, use_cuda=use_gpu, multi_gpu=False)
+        if len(self.devices) > 1:
+            logger.warning(
+                f"Multiple devices are not supported in {self.__class__.__name__} inference, "
+                f"using the first device {self.devices[0]}."
+            )
 
         self.tokenizer = RagTokenizer.from_pretrained(model_name_or_path, use_auth_token=use_auth_token)
 
@@ -165,7 +176,7 @@ class RAGenerator(BaseGenerator):
             for i in range(len(texts))
         ]
 
-        contextualized_inputs = self.tokenizer.generator.batch_encode_plus(
+        contextualized_inputs = self.tokenizer.generator(
             rag_input_strings,
             max_length=self.model.config.max_combined_length,
             return_tensors=return_tensors,
@@ -177,7 +188,7 @@ class RAGenerator(BaseGenerator):
             self.devices[0]
         )
 
-    def _prepare_passage_embeddings(self, docs: List[Document], embeddings: List[numpy.ndarray]) -> torch.Tensor:
+    def _prepare_passage_embeddings(self, docs: List[Document], embeddings: numpy.ndarray) -> torch.Tensor:
 
         # If document missing embedding, then need embedding for all the documents
         is_embedding_required = embeddings is None or any(embedding is None for embedding in embeddings)
@@ -207,16 +218,16 @@ class RAGenerator(BaseGenerator):
         :return: Generated answers plus additional infos in a dict like this:
 
         ```python
-        |     {'query': 'who got the first nobel prize in physics',
-        |      'answers':
-        |          [{'query': 'who got the first nobel prize in physics',
-        |            'answer': ' albert einstein',
-        |            'meta': { 'doc_ids': [...],
-        |                      'doc_scores': [80.42758 ...],
-        |                      'doc_probabilities': [40.71379089355469, ...
-        |                      'content': ['Albert Einstein was a ...]
-        |                      'titles': ['"Albert Einstein"', ...]
-        |      }}]}
+        {'query': 'who got the first nobel prize in physics',
+         'answers':
+             [{'query': 'who got the first nobel prize in physics',
+               'answer': ' albert einstein',
+               'meta': { 'doc_ids': [...],
+                         'doc_scores': [80.42758 ...],
+                         'doc_probabilities': [40.71379089355469, ...
+                         'content': ['Albert Einstein was a ...]
+                         'titles': ['"Albert Einstein"', ...]
+         }}]}
         ```
         """
         torch.set_grad_enabled(False)
@@ -227,7 +238,7 @@ class RAGenerator(BaseGenerator):
 
         if top_k > self.num_beams:
             top_k = self.num_beams
-            logger.warning(f"top_k value should not be greater than num_beams, hence setting it to {top_k}")
+            logger.warning("top_k value should not be greater than num_beams, hence setting it to %s", top_k)
 
         # Flatten the documents so easy to reference
         flat_docs_dict = self._flatten_docs(documents)
@@ -297,31 +308,31 @@ class Seq2SeqGenerator(BaseGenerator):
 
     **Example**
 
-    ```python
-    |     query = "Why is Dothraki language important?"
-    |
-    |     # Retrieve related documents from retriever
-    |     retrieved_docs = retriever.retrieve(query=query)
-    |
-    |     # Now generate answer from query and retrieved documents
-    |     generator.predict(
-    |        query=query,
-    |        documents=retrieved_docs,
-    |        top_k=1
-    |     )
-    |
-    |     # Answer
-    |
-    |     {'query': 'who got the first nobel prize in physics',
-    |      'answers':
-    |          [{'query': 'who got the first nobel prize in physics',
-    |            'answer': ' albert einstein',
-    |            'meta': { 'doc_ids': [...],
-    |                      'doc_scores': [80.42758 ...],
-    |                      'doc_probabilities': [40.71379089355469, ...
-    |                      'content': ['Albert Einstein was a ...]
-    |                      'titles': ['"Albert Einstein"', ...]
-    |      }}]}
+     ```python
+     query = "Why is Dothraki language important?"
+
+     # Retrieve related documents from retriever
+     retrieved_docs = retriever.retrieve(query=query)
+
+     # Now generate answer from query and retrieved documents
+     generator.predict(
+        query=query,
+        documents=retrieved_docs,
+        top_k=1
+     )
+
+     # Answer
+
+     {'query': 'who got the first nobel prize in physics',
+      'answers':
+          [{'query': 'who got the first nobel prize in physics',
+            'answer': ' albert einstein',
+            'meta': { 'doc_ids': [...],
+                      'doc_scores': [80.42758 ...],
+                      'doc_probabilities': [40.71379089355469, ...
+                      'content': ['Albert Einstein was a ...]
+                      'titles': ['"Albert Einstein"', ...]
+      }}]}
     ```
     """
 
@@ -338,6 +349,7 @@ class Seq2SeqGenerator(BaseGenerator):
         use_gpu: bool = True,
         progress_bar: bool = True,
         use_auth_token: Optional[Union[str, bool]] = None,
+        devices: Optional[List[Union[str, torch.device]]] = None,
     ):
         """
         :param model_name_or_path: a HF model name for auto-regressive language model like GPT2, XLNet, XLM, Bart, T5 etc
@@ -357,6 +369,10 @@ class Seq2SeqGenerator(BaseGenerator):
                                 `transformers-cli login` (stored in ~/.huggingface) will be used.
                                 Additional information can be found here
                                 https://huggingface.co/transformers/main_classes/model.html#transformers.PreTrainedModel.from_pretrained
+        :param devices: List of torch devices (e.g. cuda, cpu, mps) to limit inference to specific devices.
+                        A list containing torch device objects and/or strings is supported (For example
+                        [torch.device('cuda:0'), "mps", "cuda:1"]). When specifying `use_gpu=False` the devices
+                        parameter is not used and a single cpu device is used for inference.
         """
         super().__init__(progress_bar=progress_bar)
         self.model_name_or_path = model_name_or_path
@@ -366,11 +382,16 @@ class Seq2SeqGenerator(BaseGenerator):
 
         if top_k > self.num_beams:
             top_k = self.num_beams
-            logger.warning(f"top_k value should not be greater than num_beams, hence setting it to {num_beams}")
+            logger.warning("top_k value should not be greater than num_beams, hence setting it to %s", num_beams)
 
         self.top_k = top_k
 
-        self.devices, _ = initialize_device_settings(use_cuda=use_gpu, multi_gpu=False)
+        self.devices, _ = initialize_device_settings(devices=devices, use_cuda=use_gpu, multi_gpu=False)
+        if len(self.devices) > 1:
+            logger.warning(
+                f"Multiple devices are not supported in {self.__class__.__name__} inference, "
+                f"using the first device {self.devices[0]}."
+            )
 
         Seq2SeqGenerator._register_converters(model_name_or_path, input_converter)
 
@@ -391,7 +412,7 @@ class Seq2SeqGenerator(BaseGenerator):
             cls._model_input_converters[model_name_or_path] = custom_converter
 
     @classmethod
-    def _get_converter(cls, model_name_or_path: str):
+    def _get_converter(cls, model_name_or_path: str) -> Optional[Callable]:
         return cls._model_input_converters.get(model_name_or_path)
 
     def predict(self, query: str, documents: List[Document], top_k: Optional[int] = None) -> Dict:
@@ -413,10 +434,10 @@ class Seq2SeqGenerator(BaseGenerator):
 
         if top_k > self.num_beams:
             top_k = self.num_beams
-            logger.warning(f"top_k value should not be greater than num_beams, hence setting it to {top_k}")
+            logger.warning("top_k value should not be greater than num_beams, hence setting it to %s", top_k)
 
-        converter: Callable = Seq2SeqGenerator._get_converter(self.model_name_or_path)
-        if not converter:
+        converter: Optional[Callable] = Seq2SeqGenerator._get_converter(self.model_name_or_path)
+        if converter is None:
             raise KeyError(
                 f"Seq2SeqGenerator doesn't have input converter registered for {self.model_name_or_path}. "
                 f"Provide custom converter for {self.model_name_or_path} in Seq2SeqGenerator initialization"
