@@ -304,7 +304,7 @@ class _RetribertEmbeddingEncoder(_BaseEmbeddingEncoder):
 
         for i, batch in enumerate(tqdm(dataloader, desc=f"Creating Embeddings", unit=" Batches", disable=disable_tqdm)):
             batch = {key: batch[key].to(self.embedding_model.device) for key in batch}
-            with torch.no_grad():
+            with torch.inference_mode():
                 q_reps = (
                     self.embedding_model.embed_questions(
                         input_ids=batch["input_ids"], attention_mask=batch["padding_mask"]
@@ -331,7 +331,7 @@ class _RetribertEmbeddingEncoder(_BaseEmbeddingEncoder):
 
         for i, batch in enumerate(tqdm(dataloader, desc=f"Creating Embeddings", unit=" Batches", disable=disable_tqdm)):
             batch = {key: batch[key].to(self.embedding_model.device) for key in batch}
-            with torch.no_grad():
+            with torch.inference_mode():
                 q_reps = (
                     self.embedding_model.embed_answers(
                         input_ids=batch["input_ids"], attention_mask=batch["padding_mask"]
@@ -400,10 +400,22 @@ class _OpenAIEmbeddingEncoder(_BaseEmbeddingEncoder):
         model_class: str = next(
             (m for m in ["ada", "babbage", "davinci", "curie"] if m in retriever.embedding_model), "babbage"
         )
-        self.query_model_encoder_engine = f"text-search-{model_class}-query-001"
-        self.doc_model_encoder_engine = f"text-search-{model_class}-doc-001"
+        self._setup_encoding_models(model_class, retriever.embedding_model)
+
         self.tokenizer = AutoTokenizer.from_pretrained("gpt2")
 
+    def _setup_encoding_models(self, model_class: str, model_name: str):
+        """
+        Setup the encoding models for the retriever.
+        """
+        # new generation of embedding models (December 2022), we need to specify the full name
+        if "text-embedding" in model_name:
+            self.query_encoder_model = model_name
+            self.doc_encoder_model = model_name
+        else:
+            self.query_encoder_model = f"text-search-{model_class}-query-001"
+            self.doc_encoder_model = f"text-search-{model_class}-doc-001"
+
     def _ensure_text_limit(self, text: str) -> str:
         """
         Ensure that length of the text is within the maximum length of the model.
@@ -449,10 +461,10 @@ class _OpenAIEmbeddingEncoder(_BaseEmbeddingEncoder):
         return np.concatenate(all_embeddings)
 
     def embed_queries(self, queries: List[str]) -> np.ndarray:
-        return self.embed_batch(self.query_model_encoder_engine, queries)
+        return self.embed_batch(self.query_encoder_model, queries)
 
     def embed_documents(self, docs: List[Document]) -> np.ndarray:
-        return self.embed_batch(self.doc_model_encoder_engine, [d.content for d in docs])
+        return self.embed_batch(self.doc_encoder_model, [d.content for d in docs])
 
     def train(
         self,
@@ -477,7 +489,14 @@ class _CohereEmbeddingEncoder(_BaseEmbeddingEncoder):
         self.api_key = retriever.api_key
         self.batch_size = min(16, retriever.batch_size)
         self.progress_bar = retriever.progress_bar
-        self.model: str = next((m for m in ["small", "medium", "large"] if m in retriever.embedding_model), "large")
+        self.model: str = next(
+            (
+                m
+                for m in ["small", "medium", "large", "multilingual-22-12", "finance-sentiment"]
+                if m in retriever.embedding_model
+            ),
+            "multilingual-22-12",
+        )
         self.tokenizer = AutoTokenizer.from_pretrained("gpt2")
 
     def _ensure_text_limit(self, text: str) -> str:
